/**
 * Die Klasse Baum implementiert einen Knoten eines binaeren (Such)Baumes.
 * 
 * Umgebung: JDK 1.3, Windows 2000
 * Erstellt: 17.12.2001
 */
public class Baum {

  /** Die Wurzel des Baumes */
  private Knoten wurzel;
  
  /**
   * Erzeugt einen leeren Baum.
   */
  public Baum() {
  }
  
  /**
   * Erzeugt einen Baum mit der Wurzel, die die Zahl wert enthaelt.
   * Die Wurzel hat keine Nachfolgerknoten.
   *
   * @param wert die Zahl, die der Wurzelknoten speichert. 
   */
  public Baum(int wert) {
    wurzel = new Knoten(wert);
  }
  
  /**
   * Fuegt im Baum einen neuen Knoten mit der Zahl wert ein.
   *
   * @param wert die Zahl, die in einem neuen Knoten des Baumes gespeichert wird.
   */
  public void addBlatt(int wert) {
    wurzel = addBlatt(wert, wurzel);
  }
  
  /**
   * Fuegt im Baum, beginned mit dem Knoten knoten, einen neuen Knoten mit der Zahl wert ein.
   *
   * @param wert die Zahl, die in einem neuen Knoten des Baumes gespeichert wird.
   * @param knoten der aktuelle Knoten des Baumes.
   * 
   * @return der neue Knoten, der die Zahl wert enthaelt und seine Nachfolger entsprechend
   *         der Struktur des Baumes gesetzt sind. 
   */
  private Knoten addBlatt(int wert, Knoten knoten) {
    if (knoten == null) return new Knoten(wert);
    else if (wert < knoten.getWert())
      knoten.setLinks(addBlatt(wert, knoten.getLinks()));
    else
      knoten.setRechts(addBlatt(wert, knoten.getRechts()));
    return knoten;  
  }
  
  /**
   * Gibt den Baum mit preorder-Startegie auf dem Bildschirm aus.
   */
  public void pre_order() {
    System.out.println("");
    pre_order(wurzel);
  }
  
  /**
   * Gibt den Baum, beginned mit dem Knoten knoten, mit preorder-Startegie auf dem Bildschirm aus.
   *
   * @param knoten die Wurzel des Baumes, der mit preorder-Startegie auf dem Bildschirm ausgegeben wird.
   */
  private void pre_order(Knoten knoten) {
    if (knoten != null) {
      System.out.print(knoten.getWert() + " ");
      pre_order(knoten.getLinks());
      pre_order(knoten.getRechts());
    }
  }
  
  /**
   * Gibt den Baum mit inorder-Startegie auf dem Bildschirm aus.
   */
  public void in_order() {
    System.out.println("");
    in_order(wurzel);
  }
  
  /**
   * Gibt den Baum, beginned mit dem Knoten knoten, mit inorder-Startegie auf dem Bildschirm aus.
   *
   * @param knoten die Wurzel des Baumes, der mit inorder-Startegie auf dem Bildschirm ausgegeben wird.
   */
  private void in_order(Knoten knoten) {
    if (knoten != null) {
      in_order(knoten.getLinks());
      System.out.print(knoten.getWert() + " ");
      in_order(knoten.getRechts());
    }
  }
  
  /**
   * Gibt den Baum mit postorder-Startegie auf dem Bildschirm aus.
   */
  public void post_order() {
    System.out.println("");
    post_order(wurzel);
  }
  
  /**
   * Gibt den Baum, beginned mit dem Knoten knoten, mit postorder-Startegie auf dem Bildschirm aus.
   *
   * @param knoten die Wurzel des Baumes, der mit postorder-Startegie auf dem Bildschirm ausgegeben wird.
   */
  private void post_order(Knoten knoten) {
    if (knoten != null) {
      post_order(knoten.getLinks());
      post_order(knoten.getRechts());
      System.out.print(knoten.getWert() + " ");
    }
  }
  
  /**
   * Ueberpreuft, ob der Baum leer ist.
   */
  public boolean istLeer() {
    return wurzel == null;
  }

  /**
   * Liefert die Hoehe des Baumes zurueck.
   * 
   * @return die Hoehe des Baumes.
   */
  public int gibHoehe() {
    return gibHoehe(wurzel);
  }
  
  /**
   * Liefert die Hoehe des Baumes beginnend mit dem Knoten knoten zurueck.
   * 
   * @param knoten Wurzel des Baumes, dessen Hoehe berechnet wird.
   *
   * @return die Hoehe des Baumes beginned mit dem Knoten knoten.
   */
  private static int gibHoehe(Knoten knoten) {
    if (knoten == null) return 0;
    else return 1 + Math.max(gibHoehe(knoten.getLinks()), gibHoehe(knoten.getRechts()));
  }

  /**
   * Liefert die Anzahl der Knoten zurueck.
   *
   * @return die Anzahl der Knoten des Baumes.
   */
  public int gibAnzahlKnoten() {
    return gibAnzahlKnoten(wurzel);
  }

  /**
   * Liefert die Anzahl der Knoten des Baumes mit der Wurzel knoten zurueck.
   *
   * @param knoten ein Knoten (die Wurzel eines (Unter)Baumes) 
   *
   * @return die Anzahl der Knoten des Baumes mit der Wurzel knoten.
   */  
  private static int gibAnzahlKnoten(Knoten knoten) {
    if (knoten == null) return 0;
    else return 1 + gibAnzahlKnoten(knoten.getLinks()) +  gibAnzahlKnoten(knoten.getRechts());
  }

  /**
   * Ueberprueft, ob der Baum einen Knoten mit der Zahl wert enthaelt.
   *
   * @param wert die Zahl, nach der in dem Baum gesucht wird.
   *
   * @return true, falls der Baum einen Knoten mit der Zahl wert enthaelt;
   *         false, sonst.
   */
  public boolean enthaelt(int wert) {
    return enthaelt(wert, wurzel);
  }

  /**
   * Ueberprueft, ob der Baum mit der Wurzel knoten einen Knoten mit der Zahl wert enthaelt.
   *
   * @param wert eine Zahl, nach der in den Knoten des Baumes gesucht wird.
   * @param knoten die Wurzel des Baumes, in dem nach der Zahl wert gesucht wird.
   *
   * @return true, falls der Baum einen Knoten mit der Zahl wert enthaelt;
   *         false, sonst.
   */
  private static boolean enthaelt(int wert, Knoten knoten) {
    if (knoten == null) return false;
    else if (knoten.getWert() == wert) return true;
    else return enthaelt(wert, knoten.getLinks()) || enthaelt(wert, knoten.getRechts());
  }

}